﻿using UnityEngine;
using System.Collections.Generic;

namespace vamtails
{
	public class Vamtails : MVRScript
	{
		protected JSONStorableFloat jsonPosX;
		protected JSONStorableFloat jsonPosY;
		protected JSONStorableFloat jsonPosZ;

		protected JSONStorableFloat jsonRotX;
		protected JSONStorableFloat jsonRotY;
		protected JSONStorableFloat jsonRotZ;

		protected JSONStorableString infoText;
		protected JSONStorableString infoTextRight;

		protected JSONStorableStringChooser tailChoice;
		protected JSONStorableStringChooser tailPrefabChoice;
		protected UIDynamicButton setPosBTN;

		protected float sliderPosX;
		protected float sliderPosY;
		protected float sliderPosZ;

		protected float sliderRotX;
		protected float sliderRotY;
		protected float sliderRotZ;

		protected JSONStorableColor tailColor;
		HSVColor hsvTailColor;
		HSVColor prevTailColor;

		// IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
		// some reason

		// IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
		// is called right after creation
		public override void Init()
		{
			try
			{
				pluginLabelJSON.val = "Vamtails";
				// put init code in here

				jsonPosX = new JSONStorableFloat("X offset", 0.0f, -1.0f, 1.0f);
				jsonPosY = new JSONStorableFloat("Y offset", 0.0f, -1.0f, 1.0f);
				jsonPosZ = new JSONStorableFloat("Z offset", 0.0f, -1.0f, 1.0f);

				jsonRotX = new JSONStorableFloat("X rotation", 0.0f, 0.0f, 359.0f);
				jsonRotY = new JSONStorableFloat("Y rotation", 0.0f, 0.0f, 359.0f);
				jsonRotZ = new JSONStorableFloat("Z rotation", 0.0f, 0.0f, 359.0f);

				infoText = new JSONStorableString("Info","To change the color of a tail you need to save the tails position first.\n\n" +
					"After that you need to select the tail atom and tail prefab. \n\n" +
					"The prefab should look like this: vamtails-tail-ANIMAL(CLONE)");

				infoTextRight = new JSONStorableString("Info", "Saving the position attaches the tail to the hip of the person who the script is on with the specified offset and rotation.\n\n" +
					"If the atom doesn't show up in the list, reload the script.");
				
				hsvTailColor = new HSVColor();
				hsvTailColor.H = 0f;
				hsvTailColor.S = 0f;
				hsvTailColor.V = 1f;
				tailColor = new JSONStorableColor("Tail color", hsvTailColor);
				prevTailColor = hsvTailColor;
				RegisterColor(tailColor);
				
				RegisterFloat(jsonPosX);
				RegisterFloat(jsonPosY);
				RegisterFloat(jsonPosZ);

				RegisterFloat(jsonRotX);
				RegisterFloat(jsonRotY);
				RegisterFloat(jsonRotZ);

				CreateSlider(jsonPosX, true);
				CreateSlider(jsonPosY, true);
				CreateSlider(jsonPosZ, true);

				CreateSlider(jsonRotX, true);
				CreateSlider(jsonRotY, true);
				CreateSlider(jsonRotZ, true);

				tailChoice = new JSONStorableStringChooser("Tail atom",
					 SuperController.singleton.GetAtomUIDs(), "", "Tail atom");
				RegisterStringChooser(tailChoice);
				CreateFilterablePopup(tailChoice);

				List<string> tailPrefabs = getTailPrefabs(containingAtom.reParentObject);
				tailPrefabChoice = new JSONStorableStringChooser("Tail prefab",
					 tailPrefabs, "", "Tail prefab");
				RegisterStringChooser(tailPrefabChoice);
				CreateFilterablePopup(tailPrefabChoice);

				// initialize to whatever the slider is now.
				sliderPosX = jsonPosX.val;
				sliderPosY = jsonPosY.val;
				sliderPosZ = jsonPosZ.val;

				sliderRotX = jsonRotX.val;
				sliderRotY = jsonRotY.val;
				sliderRotZ = jsonRotZ.val;

				//SuperController.LogMessage("Vamtails Loaded");

				// create custom JSON storable params here if you want them to be stored with scene JSON
				// types are JSONStorableFloat, JSONStorableBool, JSONStorableString, JSONStorableStringChooser
				// JSONStorableColor

				setPosBTN = CreateButton("Save tail position", false);
				setPosBTN.height = 120;
				setPosBTN.button.onClick.AddListener(delegate ()
				{
					foreach (Atom atom in GetSceneAtoms())
					{
						if (atom.name == tailChoice.val)
						{
							//SuperController.LogMessage(jsonFloat.ToString());
							
							atom.mainController.SelectLinkToRigidbody(null);
							atom.SelectAtomParent(null);

							updateTailPos();

							atom.SelectAtomParent(containingAtom);

							UnityEngine.Rigidbody hipRB = new UnityEngine.Rigidbody();
							foreach (UnityEngine.Rigidbody rb in containingAtom.rigidbodies)
							{
								if (rb.name == "hip")
								{
									hipRB = rb;
								}
							}
							atom.mainController.SelectLinkToRigidbody(hipRB);
						}
					}
				});

				CreateColorPicker(tailColor);

				UIDynamicTextField infoTextField = CreateTextField(infoText);
				UIDynamicTextField infoTextFieldRight = CreateTextField(infoTextRight, true);

				infoTextField.height = 400;
				infoTextFieldRight.height = 400;
			}
			catch (System.Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start()
		{
			try
			{
				
			}
			catch (System.Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update()
		{
			try
			{
				// put code in here
				if (jsonPosX.val != sliderPosX) // check if slider was changed since last frame
				{
					sliderPosX = jsonPosX.val; // update our tracking value
					updateTailPos();
				}
				if (jsonPosY.val != sliderPosY) // check if slider was changed since last frame
				{
					sliderPosY = jsonPosY.val; // update our tracking value
					updateTailPos();
				}
				if (jsonPosZ.val != sliderPosZ) // check if slider was changed since last frame
				{
					sliderPosZ = jsonPosZ.val; // update our tracking value
					updateTailPos();
				}

				if (jsonRotX.val != sliderRotX) // check if slider was changed since last frame
				{
					sliderRotX = jsonRotX.val; // update our tracking value
					updateTailPos();
				}
				if (jsonRotY.val != sliderRotY) // check if slider was changed since last frame
				{
					sliderRotY = jsonRotY.val; // update our tracking value
					updateTailPos();
				}
				if (jsonRotZ.val != sliderRotZ) // check if slider was changed since last frame
				{
					sliderRotZ = jsonRotZ.val; // update our tracking value
					updateTailPos();
				}

				if (!tailColor.val.Equals(prevTailColor)) // check if slider was changed since last frame
				{
					prevTailColor = tailColor.val; // update our tracking value
					updateTailColor();
				}
				tailPrefabChoice.choices = getTailPrefabs(containingAtom.reParentObject);
				tailChoice.choices = SuperController.singleton.GetAtomUIDs();
			}
			catch (System.Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void updateTailColor()
		{
			foreach (Atom atom in GetSceneAtoms())
			{
				if (atom.name == tailChoice.val)
				{
					Transform tailTR = getObjectRoot(containingAtom.reParentObject, tailPrefabChoice.val);

					if (tailTR != null)
                    {
						tailTR.GetComponent<Renderer>().material.SetColor("_Color", Color.HSVToRGB(tailColor.val.H, tailColor.val.S, tailColor.val.V));
					}
				}
			}
		}

		List<string> getTailPrefabs(Transform parent)
        {
			List<string> tails = new List<string>();

			foreach (Transform child in parent)
			{
				//SuperController.LogMessage(child.name);
				if (child.name.Contains("vamtails-tail"))
				{
					tails.Add(child.name);
				}
				else
				{
					List<string> childSearch = getTailPrefabs(child);
					tails.AddRange(childSearch);
				}
			}
			return tails;
		}



		private Transform getObjectRoot(Transform parent, string name)
		{
			foreach (Transform child in parent)
			{
				if (child.name.Contains(name))
				{
					return child;
				}
				else
				{
					Transform childSearch = getObjectRoot(child, name);
					if (childSearch != null) return childSearch;
				}
			}

			return null;
		}

		void updateTailPos()
        {
			foreach (Atom atom in GetSceneAtoms())
			{
				if (atom.name == tailChoice.val)
				{
					atom.mainController.SelectLinkToRigidbody(null);
					atom.SelectAtomParent(null);

					Vector3 parentPos = containingAtom.mainController.transform.position;
					Vector3 parentRot = containingAtom.mainController.transform.eulerAngles;

					atom.mainController.transform.position = parentPos + new UnityEngine.Vector3(sliderPosX, sliderPosY, sliderPosZ);
					atom.mainController.transform.rotation = Quaternion.Euler(parentRot + new Vector3(sliderRotX,sliderRotY,sliderRotZ));
				}
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate()
		{
			try
			{
				// put code in here
			}
			catch (System.Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy()
		{
		}

	}
}